package com.develou.p4_preferencias_con_sharedpreferences

import android.content.Intent
import android.os.Bundle
import android.view.Menu
import android.view.MenuItem
import android.widget.TextView
import androidx.appcompat.app.AppCompatActivity
import androidx.preference.PreferenceManager

class MainActivity : AppCompatActivity() {
    private lateinit var businessName: TextView
    private lateinit var currency: TextView
    private lateinit var distance: TextView
    private lateinit var accountSummary: TextView
    private lateinit var latestEvents: TextView
    private lateinit var themeText: TextView
    private lateinit var animations: TextView
    private lateinit var collectionsDesign: TextView


    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_main)
        setUpUI()
        loadPrefs()
    }

    private fun setUpUI() {
        businessName = findViewById(R.id.business_name)
        currency = findViewById(R.id.currency)
        distance = findViewById(R.id.distance)
        accountSummary = findViewById(R.id.account_summary)
        latestEvents = findViewById(R.id.latest_events)
        themeText = findViewById(R.id.theme)
        animations = findViewById(R.id.animations)
        collectionsDesign = findViewById(R.id.collections_design)
    }

    private fun loadPrefs() {
        PreferenceManager.getDefaultSharedPreferences(this).let { store ->
            businessName.text = getString(
                R.string.business_name,
                store.getString("businessName", "")
            )
            currency.text = getString(
                R.string.currency,
                store.getString("currency", "")
            )
            distance.text = getString(
                R.string.distance,
                store.getInt("distance", 0)
            )
            accountSummary.text =
                getString(
                    R.string.account_summary,
                    store.getStringSet("accountSummary", emptySet())
                )

            val eventsActive = store.getBoolean("latestEvents", false)
            var string = getString(R.string.latest_event, eventsActive)
            if (eventsActive)
                string += getString(
                    R.string.latest_events_order,
                    store.getString("latestEventsOrder", "")
                )
            latestEvents.text = string

            themeText.text = getString(
                R.string.theme,
                store.getString("theme", "Claro")
            )
            animations.text = getString(
                R.string.animations,
                store.getBoolean("animations", false)
            )
            collectionsDesign.text = getString(
                R.string.collections_design,
                store.getString("collectionsDesign", "Lista")
            )
        }
    }

    override fun onCreateOptionsMenu(menu: Menu?): Boolean {
        menuInflater.inflate(R.menu.main_menu, menu)
        return true
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        if (R.id.settings == item.itemId) {
            toSettings()
            return true
        }
        return super.onOptionsItemSelected(item)
    }

    private fun toSettings() {
        val intent = Intent(this, SettingsActivity::class.java)
        startActivity(intent)
    }
}