package com.develou.p4_preferencias_con_sharedpreferences

import android.content.SharedPreferences
import android.os.Bundle
import android.util.Log
import android.widget.Toast
import androidx.preference.EditTextPreference
import androidx.preference.MultiSelectListPreference
import androidx.preference.Preference
import androidx.preference.PreferenceFragmentCompat
import com.develou.p3_categorias_de_preferencias.BuildConfig

class SettingsFragment : PreferenceFragmentCompat(),
    SharedPreferences.OnSharedPreferenceChangeListener {

    override fun onCreatePreferences(savedInstanceState: Bundle?, rootKey: String?) {
        setPreferencesFromResource(R.xml.preferences, rootKey)

        setUpBuildVersion()
        setUpAccountSummary()
        setUpBusinessName()
    }

    private fun setUpBuildVersion() {
        val buildVersion: Preference? = findPreference("buildVersion")

        buildVersion?.summary = com.develou.p3_categorias_de_preferencias.BuildConfig.VERSION_NAME

        buildVersion?.setOnPreferenceClickListener {
            Log.d("SettingsFragment", "\"${it.title}\" fue clickeada")
            false
        }
    }

    private fun setUpAccountSummary() {
        val accountSummary: MultiSelectListPreference? = findPreference("accountSummary")
        accountSummary?.run {
            setSummary(values) // Texto secundario inicial
            setOnPreferenceChangeListener { _, newValue ->
                val newValues = newValue as Set<String>
                accountSummary.setSummary(newValues) // Texto secundario al cambiar
                true
            }
        }
    }

    fun MultiSelectListPreference.setSummary(newValues: Set<String>) {
        summary = if (newValues.isEmpty())
            "Sin envío"
        else
            newValues.joinToString(", ") { value ->
                entries[findIndexOfValue(value)]
            }
    }

    private fun setUpBusinessName() {
        val businessName = findPreference<EditTextPreference>("businessName")
        businessName?.setOnPreferenceChangeListener { _, newValue ->
            if (newValue.toString().length >= 10)
                true
            else {
                Toast.makeText(
                    requireActivity(),
                    "El nombre debe tener más de 10 caracteres",
                    Toast.LENGTH_SHORT
                ).show()
                false
            }
        }
    }

    override fun onSharedPreferenceChanged(sharedPreferences: SharedPreferences, key: String) {
        Toast.makeText(
            requireActivity(),
            "Preferencia cambiada: $key",
            Toast.LENGTH_SHORT
        ).show()
    }

    override fun onResume() {
        super.onResume()
        preferenceManager.sharedPreferences.registerOnSharedPreferenceChangeListener(this)
    }

    override fun onPause() {
        super.onPause()
        preferenceManager.sharedPreferences.unregisterOnSharedPreferenceChangeListener(this)
    }
}

