package com.develou.p5_preferencias_con_preferencedatastore.preference

import androidx.preference.PreferenceDataStore
import com.develou.p5_preferencias_con_preferencedatastore.room.SettingsDatabase
import com.develou.p5_preferencias_con_preferencedatastore.room.entity.UserSetting
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import kotlinx.coroutines.runBlocking

class SettingsDataStore(
    private val db: SettingsDatabase,
    private val appPrefs: AppPreferences
) : PreferenceDataStore() {

    override fun putString(key: String, value: String?) {
        putSettingValue(key, value)
    }

    override fun getString(key: String, defValue: String?): String? {
        return getSettingValue(key, defValue.orEmpty())
    }

    override fun putStringSet(key: String, values: MutableSet<String>?) {
        putSettingValue(key, values.multiSelectToString())
    }

    override fun getStringSet(key: String, defValues: Set<String>?): Set<String> {
        val value = getSettingValue(key, defValues.multiSelectToString())
        return value.stringToMultiSelect()
    }

    override fun putInt(key: String, value: Int) {
        putSettingValue(key, value)
    }

    override fun getInt(key: String, defValue: Int): Int {
        return getSettingValue(key, defValue.toString()).toInt()
    }

    override fun putBoolean(key: String, value: Boolean) {
        putSettingValue(key, value)
    }

    override fun getBoolean(key: String, defValue: Boolean): Boolean {
        return getSettingValue(key, defValue.toString()).toBoolean()
    }

    private fun putSettingValue(key: String, value: Any?) {
        CoroutineScope(Dispatchers.IO).launch {
            val userId = appPrefs.getUser()
            val settingId = db.settingDao().getSettingId(key)!!
            val setting = UserSetting(userId, settingId, value.toString())
            db.userSettingDao().insert(setting)
        }
    }

    private fun getSettingValue(key: String, defValue: String) = runBlocking {
        val userId = appPrefs.getUser()
        val settingId = db.settingDao().getSettingId(key)!!
        val value = db.userSettingDao().getValue(userId, settingId)

        // Si no existe aún el valor del ajuste, guardamos el valor por defecto
        if (value == null) {
            db.userSettingDao().insert(UserSetting(userId, settingId, defValue))
            defValue
        } else
            value
    }
}

private fun Set<String>?.multiSelectToString(): String {
    return this?.joinToString(",").orEmpty()
}

private fun String?.stringToMultiSelect(): Set<String> {
    return this?.split(",")?.toSet() ?: setOf()
}