package com.develou.p5_preferencias_con_preferencedatastore.room

import android.content.Context
import androidx.room.Database
import androidx.room.Room
import androidx.room.RoomDatabase
import com.develou.p5_preferencias_con_preferencedatastore.room.dao.SettingDao
import com.develou.p5_preferencias_con_preferencedatastore.room.dao.UserDao
import com.develou.p5_preferencias_con_preferencedatastore.room.dao.UserSettingsDao
import com.develou.p5_preferencias_con_preferencedatastore.room.entity.Setting
import com.develou.p5_preferencias_con_preferencedatastore.room.entity.User
import com.develou.p5_preferencias_con_preferencedatastore.room.entity.UserSetting

@Database(
    entities = [User::class, UserSetting::class, Setting::class],
    version = 1
)
abstract class SettingsDatabase : RoomDatabase() {

    abstract fun userDao(): UserDao
    abstract fun userSettingDao(): UserSettingsDao
    abstract fun settingDao(): SettingDao

    companion object {
        @Volatile
        private var INSTANCE: SettingsDatabase? = null

        fun getInstance(context: Context): SettingsDatabase =
            INSTANCE ?: synchronized(this) {
                INSTANCE ?: buildDatabase(context).also { db ->
                    INSTANCE = db
                }
            }

        private fun buildDatabase(context: Context): SettingsDatabase {
            return Room.databaseBuilder(
                context,
                SettingsDatabase::class.java,
                "settings.db"
            ).build()
        }
    }
}