package com.develou.p5_preferencias_con_preferencedatastore.ui

import android.content.Intent
import android.os.Bundle
import android.view.Menu
import android.view.MenuItem
import android.widget.RadioGroup
import android.widget.TextView
import androidx.appcompat.app.AppCompatActivity
import com.develou.p5_preferencias_con_preferencedatastore.R
import com.develou.p5_preferencias_con_preferencedatastore.SettingsApplication
import com.develou.p5_preferencias_con_preferencedatastore.room.entity.Setting
import com.develou.p5_preferencias_con_preferencedatastore.room.entity.User
import kotlinx.coroutines.*

class MainActivity : AppCompatActivity() {
    private lateinit var usersContainer: RadioGroup
    private lateinit var businessName: TextView
    private lateinit var currency: TextView
    private lateinit var distance: TextView
    private lateinit var accountSummary: TextView
    private lateinit var latestEvents: TextView
    private lateinit var themeText: TextView
    private lateinit var animations: TextView
    private lateinit var collectionsDesign: TextView


    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_main)
        setUpUI()

        MainScope().launch {
            initializeDatabase()
        }
    }

    private suspend fun initializeDatabase() = coroutineScope {
        val data = async(Dispatchers.IO) {
            val db = (application as SettingsApplication).database
            val userDao = db.userDao()
            val settingDao = db.settingDao()

            userDao.insert(User(1, "usuario_1"))
            userDao.insert(User(2, "usuario_2"))

            settingDao.insert(Setting(name = "businessName"))
            settingDao.insert(Setting(name = "currency"))
            settingDao.insert(Setting(name = "distance"))
            settingDao.insert(Setting(name = "accountSummary"))
            settingDao.insert(Setting(name = "latestEvents"))
            settingDao.insert(Setting(name = "latestEventsOrder"))
            settingDao.insert(Setting(name = "theme"))
            settingDao.insert(Setting(name = "animations"))
            settingDao.insert(Setting(name = "collectionsDesign"))
        }

        withContext(Dispatchers.Main) {
            data.await()
            loadPrefs()
        }
    }

    private fun setUpUI() {
        setUpUsers()

        businessName = findViewById(R.id.business_name)
        currency = findViewById(R.id.currency)
        distance = findViewById(R.id.distance)
        accountSummary = findViewById(R.id.account_summary)
        latestEvents = findViewById(R.id.latest_events)
        themeText = findViewById(R.id.theme)
        animations = findViewById(R.id.animations)
        collectionsDesign = findViewById(R.id.collections_design)
    }

    private fun setUpUsers() {
        val appPrefs = (application as SettingsApplication).appPrefs

        // Marcar usuario inicial
        usersContainer = findViewById(R.id.users_container)
        val radioId = if (appPrefs.getUser() == 1) R.id.user_1_button else R.id.user_2_button
        usersContainer.check(radioId)

        // Procesar eventos de cambio de usuario
        usersContainer.setOnCheckedChangeListener { _, checkedId ->
            val userId = if (checkedId == R.id.user_1_button) 1 else 2
            // Guardar usuario seleccionado
            appPrefs.saveUser(userId)

            // Actualizar vista
            loadPrefs()
        }
    }

    private fun loadPrefs() {
        ((application as SettingsApplication).settingsDataStore).let { store ->
            businessName.text = getString(
                R.string.business_name,
                store.getString("businessName", "No establecido")
            )
            currency.text = getString(
                R.string.currency,
                store.getString("currency", "cop")
            )
            distance.text = getString(
                R.string.distance,
                store.getInt("distance", 50)
            )
            accountSummary.text =
                getString(
                    R.string.account_summary,
                    store.getStringSet("accountSummary", setOf("1", "2", "3"))
                )

            val eventsActive = store.getBoolean("latestEvents", true)
            var string = getString(R.string.latest_event, eventsActive)
            if (eventsActive)
                string += getString(
                    R.string.latest_events_order,
                    store.getString("latestEventsOrder", "date")
                )
            latestEvents.text = string

            themeText.text = getString(
                R.string.theme,
                store.getString("theme", "Claro")
            )
            animations.text = getString(
                R.string.animations,
                store.getBoolean("animations", true)
            )
            collectionsDesign.text = getString(
                R.string.collections_design,
                store.getString("collectionsDesign", "Lista")
            )
        }
    }

    override fun onCreateOptionsMenu(menu: Menu?): Boolean {
        menuInflater.inflate(R.menu.main_menu, menu)
        return true
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        if (R.id.settings == item.itemId) {
            toSettings()
            return true
        }
        return super.onOptionsItemSelected(item)
    }

    private fun toSettings() {
        val intent = Intent(this, SettingsActivity::class.java)
        startActivity(intent)
    }
}